import re
from django import forms
from usermerge.validators import platform_is_selected_from_provided_list, ece_id_is_not_031YY000

# Create your forms here.
# https://docs.djangoproject.com/en/2.0/ref/forms/

# Each programmatic form from the ones defined below is based format-wise on the similarly named template form (e.g. LoginForm is based on the login_form of login.html, UserProfileEditForm is based on the user_profile_edit_form of user_profile_edit.html, etc.) and constraint-wise on the application models (see models.py).

class LoginForm(forms.Form):
    platform = forms.CharField(error_messages = {'required': 'Το όνομα πλατφόρμας πρέπει απαραίτητα να επιλεχθεί και να είναι μη-κενό!'},
                               validators = [platform_is_selected_from_provided_list])
    username = forms.CharField(max_length = 50,
                               error_messages = {'required': 'Το όνομα χρήστη πρέπει απαραίτητα να συμπληρωθεί!',
                                                 'max_length': 'Το όνομα χρήστη δεν πρέπει να υπερβαίνει τους 50 χαρακτήρες!'})
    password = forms.CharField(max_length = 50,
                               error_messages = {'required': 'Ο κωδικός πρόσβασης πρέπει απαραίτητα να συμπληρωθεί!',
                                                 'max_length': 'Ο κωδικός πρόσβασης δεν πρέπει να υπερβαίνει τους 50 χαρακτήρες!'})

    def clean(self):
        cleaned_data = super().clean()
        # If platform and username have survived the initial individual field checks (by the time this method is called, all the
        # individual field clean methods will have been run, so cleaned_data will be populated with any data that has survived so far)
        # and platform refers either to Novice or to Grader, ensure that the username format is piYYbSSS (if it is not, raise a
        # ValidationError exception with the appropriate error message and code), where YY refers to year and SSS refers to non-000
        # serial number.
        if set(('platform', 'username')).issubset(cleaned_data):
            platform = cleaned_data['platform']
            username = cleaned_data['username']
            if platform == 'Novice' or platform == 'Grader':
                if re.fullmatch('pi[0-9]{2}b[0-9]{3}', username):
                    if username[5:] == '000':
                        raise forms.ValidationError('Το όνομα χρήστη στις πλατφόρμες Novice και Grader απαγορεύεται να\n'
                                                    'είναι της μορφής piYYb000 (YY: έτος)!', code = 'novice_or_grader_username_is_piYYb000')
                else:
                    raise forms.ValidationError('Το όνομα χρήστη στις πλατφόρμες Novice και Grader πρέπει να\n'
                                                'είναι της μορφής piYYbSSS (ΥΥ: έτος, SSS: αύξων αριθμός)!',
                                                code = 'novice_or_grader_username_format_is_not_piYYbSSS')

class UserProfileEditForm(forms.Form):
    first_name = forms.CharField(max_length = 50,
                                 error_messages = {'required': 'Το όνομα πρέπει απαραίτητα να συμπληρωθεί!',
                                                   'max_length': 'Το όνομα δεν πρέπει να υπερβαίνει τους 50 χαρακτήρες!'})
    last_name = forms.CharField(max_length = 50,
                                error_messages = {'required': 'Το επώνυμο πρέπει απαραίτητα να συμπληρωθεί!',
                                                  'max_length': 'Το επώνυμο δεν πρέπει να υπερβαίνει τους 50 χαρακτήρες!'})
    ece_id = forms.RegexField(required = False, max_length = 8, regex = '031[0-9]{5}',
                              # The empty value '' (empty string) becomes None in order to be interpreted as NULL in usermergeDB.
                              empty_value = None, error_messages = {'max_length': 'Ο αριθμός μητρώου πρέπει να είναι της μορφής '
                                                                                  '031YYSSS (YY: έτος, SSS: αύξων αριθμός)!',
                                                                    'invalid': 'Ο αριθμός μητρώου πρέπει να είναι της μορφής '
                                                                               '031YYSSS (YY: έτος, SSS: αύξων αριθμός)!'},
                              validators = [ece_id_is_not_031YY000])
    email = forms.EmailField(max_length = 254,
                             error_messages = {'required': 'Το ηλεκτρονικό ταχυδρομείο πρέπει απαραίτητα να συμπληρωθεί!',
                                               'max_length': 'Το ηλεκτρονικό ταχυδρομείο δεν πρέπει να υπερβαίνει τους 254 χαρακτήρες!',
                                               'invalid': 'Το ηλεκτρονικό ταχυδρομείο πρέπει να είναι της μορφής local-part@domain που\n'
                                                          'περιγράφεται στην ιστοσελίδα https://en.wikipedia.org/wiki/Email_address !'})

class UserProfileRecoveryForm(UserProfileEditForm):
    first_name = None
    last_name = None

