from django.core.exceptions import ValidationError
from usermerge.models import Platform

# Create your validators here.
# https://docs.djangoproject.com/en/2.0/ref/validators/

# The validators of a form field are run in the context of run_validators() method when the clean() method of the field is called. If the field value is empty, i.e. None, '', [], () or {}, run_validators() will NOT run the validators (see https://docs.djangoproject.com/en/2.0/_modules/django/forms/fields/). Therefore, it can be safely assumed that the single arguments of the following validators are NEVER empty.
# For more information on the platform names that are available to select from in forms, see get_names_of_SoftLab_provided_platforms_from_DB() function of helper.py . In the case of login form, see also login_form of login.html .

def platform_is_selected_from_provided_list(platform):
    """
    Ensure that the platform (name) is selected among the ones provided in the corresponding form drop-down list.
    If it is not (e.g. it is misedited via JavaScript), raise a ValidationError exception with the appropriate error message and code.
    """
    if platform != 'SLUB' and not (platform != 'ECE-NTUA' and platform in Platform.objects.values_list('name', flat = True)):
        raise ValidationError('Το όνομα πλατφόρμας πρέπει να επιλέγεται μεταξύ εκείνων\n'
                              'που παρέχονται στην αντίστοιχη αναπτυσσόμενη λίστα!', code = 'platform_is_not_selected_from_provided_list')

def ece_id_is_not_031YY000(ece_id):
    """
    Ensure that the ece_id (format) is not 031YY000.
    If it is, raise a ValidationError exception with the appropriate error message and code.
    """
    if ece_id[5:] == '000':
        raise ValidationError('Ο αριθμός μητρώου απαγορεύεται να είναι της μορφής 031YY000 (YY: έτος)!', code = 'ece_id_is_031YY000')
